import { BaseEditor, Descendant } from 'slate';
import { HistoryEditor } from 'slate-history';
import { ReactEditor } from 'slate-react';

export type CommonElement = { align?: 'left' | 'center' | 'right' | 'justify'; children: Descendant[] };

export type BlockQuoteElement = CommonElement & { type: 'block-quote'; stage: boolean };

export type BulletedListElement = CommonElement & { type: 'bulleted-list' };

export type ContentMediumElement = CommonElement & { type: 'content-medium'; id: string; nodeId: string };

export type NFTElement = CommonElement & { type: 'nft'; id: string };

export type HeadingOneElement = CommonElement & { type: 'heading-one' };

export type HeadingTwoElement = CommonElement & { type: 'heading-two' };

export type HeadingThreeElement = CommonElement & { type: 'heading-three' };

export type IframeEmbedElement = CommonElement & { type: 'iframe-embed'; url: string; height: number };

export type LinkElement = CommonElement & { type: 'link'; url: string };

export type ListItemElement = CommonElement & { type: 'list-item' };

export type NumberedListElement = CommonElement & { type: 'numbered-list' };

export type ParagraphElement = CommonElement & { type: 'paragraph' };

export type MentionElement = CommonElement & { type: 'mention'; userId: string; nodeId: string };

type CustomElement =
  | BlockQuoteElement
  | BulletedListElement
  | ContentMediumElement
  | NFTElement
  | HeadingOneElement
  | HeadingTwoElement
  | HeadingThreeElement
  | IframeEmbedElement
  | LinkElement
  | ListItemElement
  | NumberedListElement
  | ParagraphElement
  | MentionElement;

export type CustomText = {
  bold?: boolean;
  italic?: boolean;
  underline?: boolean;
  code?: boolean;
  html?: boolean;
  text: string;
};

export type CustomEditor = BaseEditor & ReactEditor & HistoryEditor;

declare module 'slate' {
  interface CustomTypes {
    Editor: CustomEditor;
    Element: CustomElement;
    Text: CustomText;
  }
}

export type CustomNode = CustomEditor | CustomElement | CustomText;
