"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toggleBlockQuote = exports.unwrapLink = exports.wrapLink = exports.insertLink = exports.insertContentMedium = exports.insertNFT = exports.insertIframe = exports.isBlockActive = exports.isQuoteActive = void 0;
var react_1 = __importDefault(require("react"));
var slate_1 = require("slate");
var slate_react_1 = require("slate-react");
var uuid_1 = require("uuid");
var components_1 = require("./components");
var EditorIcon_1 = __importDefault(require("./EditorIcon"));
var isQuoteActive = function (editor, stage) {
    var _a = __read(slate_1.Editor.nodes(editor, {
        match: function (n) { return !slate_1.Editor.isEditor(n) && slate_1.Element.isElement(n) && n.type === 'block-quote' && n.stage === stage; },
    }), 1), match = _a[0];
    return !!match;
};
exports.isQuoteActive = isQuoteActive;
var isBlockActive = function (editor, format, blockType) {
    if (blockType === void 0) { blockType = 'type'; }
    var selection = editor.selection;
    if (!selection)
        return false;
    var _a = __read(Array.from(slate_1.Editor.nodes(editor, {
        at: slate_1.Editor.unhangRange(editor, selection),
        match: function (n) { return !slate_1.Editor.isEditor(n) && slate_1.Element.isElement(n) && n[blockType] === format; },
    })), 1), match = _a[0];
    return !!match;
};
exports.isBlockActive = isBlockActive;
var insertIframe = function (editor, url, height) {
    var iframe = { type: 'iframe-embed', url: url, height: height, children: [{ text: '' }] };
    var paragraph = { type: 'paragraph', children: [{ text: '' }] };
    slate_1.Transforms.insertNodes(editor, [iframe, paragraph]);
};
exports.insertIframe = insertIframe;
var insertNFT = function (editor, id) {
    var nft = { type: 'nft', id: id, children: [{ text: '' }] };
    var paragraph = { type: 'paragraph', children: [{ text: '' }] };
    slate_1.Transforms.insertNodes(editor, [nft, paragraph]);
};
exports.insertNFT = insertNFT;
var insertContentMedium = function (editor, id) {
    var contentMedium = { type: 'content-medium', id: id, nodeId: uuid_1.v4(), children: [{ text: '' }] };
    var paragraph = { type: 'paragraph', children: [{ text: '' }] };
    slate_1.Transforms.insertNodes(editor, [contentMedium, paragraph]);
};
exports.insertContentMedium = insertContentMedium;
var insertLink = function (editor, url) {
    if (editor.selection) {
        exports.wrapLink(editor, url);
    }
};
exports.insertLink = insertLink;
var wrapLink = function (editor, url) {
    if (exports.isBlockActive(editor, 'link')) {
        exports.unwrapLink(editor);
    }
    var selection = editor.selection;
    var isCollapsed = selection && slate_1.Range.isCollapsed(selection);
    var link = {
        type: 'link',
        url: url,
        children: isCollapsed ? [{ text: url }] : [],
    };
    if (isCollapsed) {
        slate_1.Transforms.insertNodes(editor, link);
    }
    else {
        slate_1.Transforms.wrapNodes(editor, link, { split: true });
        slate_1.Transforms.collapse(editor, { edge: 'end' });
    }
};
exports.wrapLink = wrapLink;
var unwrapLink = function (editor) {
    slate_1.Transforms.unwrapNodes(editor, {
        match: function (n) { return !slate_1.Editor.isEditor(n) && slate_1.Element.isElement(n) && n.type === 'link'; },
    });
};
exports.unwrapLink = unwrapLink;
var toggleBlockQuote = function (editor) {
    var _a = __read(slate_1.Editor.nodes(editor, {
        match: function (n) { return !slate_1.Editor.isEditor(n) && slate_1.Element.isElement(n) && n.type === 'block-quote'; },
    }), 1), match = _a[0];
    slate_1.Transforms.unwrapNodes(editor, {
        match: function (n) { return components_1.LIST_TYPES.includes((!slate_1.Editor.isEditor(n) && slate_1.Element.isElement(n) && n.type)); },
        split: true,
    });
    if (match) {
        var _b = __read(match, 1), blockQuote = _b[0];
        if (!blockQuote.stage) {
            var newProperties = { stage: true };
            slate_1.Transforms.setNodes(editor, newProperties);
        }
        else {
            var newProperties = { type: 'paragraph', stage: undefined };
            slate_1.Transforms.setNodes(editor, newProperties);
        }
    }
    else {
        var newProperties = {
            type: 'block-quote',
            stage: false,
        };
        slate_1.Transforms.setNodes(editor, newProperties);
    }
};
exports.toggleBlockQuote = toggleBlockQuote;
var toggleBlock = function (editor, format) {
    var isActive = exports.isBlockActive(editor, format, components_1.TEXT_ALIGN_TYPES.includes(format) ? 'align' : 'type');
    slate_1.Transforms.unwrapNodes(editor, {
        match: function (n) {
            return !slate_1.Editor.isEditor(n) &&
                slate_1.Element.isElement(n) &&
                components_1.LIST_TYPES.includes(n.type) &&
                !components_1.TEXT_ALIGN_TYPES.includes(format);
        },
        split: true,
    });
    var newProperties = components_1.isAlignFormat(format)
        ? { align: isActive ? undefined : format }
        : { type: isActive ? 'paragraph' : components_1.isListFormat(format) ? 'list-item' : format };
    slate_1.Transforms.setNodes(editor, newProperties);
    if (!isActive && components_1.isListFormat(format)) {
        var block = { type: format, children: [] };
        slate_1.Transforms.wrapNodes(editor, block);
    }
};
var BlockButton = function (_a) {
    var title = _a.title, format = _a.format, icon = _a.icon, orientation = _a.orientation, neverActive = _a.neverActive, onClick = _a.onClick;
    var editor = slate_react_1.useSlate();
    return (react_1.default.createElement(components_1.EditorButton, { title: title, orientation: orientation, active: !neverActive && exports.isBlockActive(editor, format, components_1.TEXT_ALIGN_TYPES.includes(format) ? 'align' : 'type'), onClick: function (event) {
            event.preventDefault();
            event.stopPropagation();
            if (onClick)
                onClick();
            else
                toggleBlock(editor, format);
        } },
        react_1.default.createElement(EditorIcon_1.default, { type: icon })));
};
exports.default = BlockButton;
//# sourceMappingURL=BlockButton.js.map