"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMentions = exports.insertMention = void 0;
var react_1 = require("react");
var slate_1 = require("slate");
var slate_react_1 = require("slate-react");
var uuid_1 = require("uuid");
var insertMention = function (editor, userId) {
    var mention = { type: 'mention', userId: userId, nodeId: uuid_1.v4(), children: [{ text: '' }] };
    slate_1.Transforms.insertNodes(editor, mention);
    slate_1.Transforms.move(editor);
};
exports.insertMention = insertMention;
var useMentions = function (editor, _a) {
    var disableMentions = _a.disableMentions, onSearchMentions = _a.onSearchMentions;
    var ref = react_1.useRef(null);
    var _b = __read(react_1.useState(null), 2), target = _b[0], setTarget = _b[1];
    var _c = __read(react_1.useState(0), 2), index = _c[0], setIndex = _c[1];
    var _d = __read(react_1.useState(''), 2), search = _d[0], setSearch = _d[1];
    var _e = __read(react_1.useState([]), 2), recentlyMentionedUsers = _e[0], setRecentlyMentionedUsers = _e[1];
    var _f = __read(react_1.useState({ users: [], total: 0 }), 2), possibleMentions = _f[0], setPossibleMentions = _f[1];
    react_1.useEffect(function () {
        if (search && search.length > 2)
            onSearchMentions && onSearchMentions(search).then(setPossibleMentions);
        else
            setPossibleMentions({ users: [], total: 0 });
    }, [search]);
    var onMention = react_1.useCallback(function (userId) {
        if (!target)
            return;
        var mentionedUsers = possibleMentions.users.find(function (user) { return user.id === userId; });
        slate_1.Transforms.select(editor, target);
        exports.insertMention(editor, userId.toString());
        setTarget(null);
        setSearch('');
        setRecentlyMentionedUsers(function (old) { return __spreadArray(__spreadArray([], __read(old)), __read([mentionedUsers].filter(Boolean))); });
    }, [target, possibleMentions]);
    var onKeyDown = react_1.useCallback(function (event) {
        if (disableMentions)
            return;
        if (target && possibleMentions.users.length > 0) {
            switch (event.key) {
                case 'ArrowDown':
                    event.preventDefault();
                    var prevIndex = index >= possibleMentions.users.length - 1 ? 0 : index + 1;
                    setIndex(prevIndex);
                    break;
                case 'ArrowUp':
                    event.preventDefault();
                    var nextIndex = index <= 0 ? possibleMentions.users.length - 1 : index - 1;
                    setIndex(nextIndex);
                    break;
                case 'Tab':
                case 'Enter':
                    event.preventDefault();
                    onMention(possibleMentions.users[index].id);
                    break;
                case 'Escape':
                    event.preventDefault();
                    setTarget(null);
                    break;
            }
        }
    }, [index, search, target, possibleMentions]);
    var onChange = react_1.useCallback(function () {
        if (disableMentions)
            return;
        var selection = editor.selection;
        if (selection && slate_1.Range.isCollapsed(selection)) {
            var _a = __read(slate_1.Range.edges(selection), 1), start = _a[0];
            var wordBefore = slate_1.Editor.before(editor, start, { unit: 'word' });
            var before = wordBefore && slate_1.Editor.before(editor, wordBefore);
            var beforeRange = before && slate_1.Editor.range(editor, before, start);
            var beforeText = beforeRange && slate_1.Editor.string(editor, beforeRange);
            var beforeMatch = beforeText && beforeText.match(/^@([a-zäöüA-ZÄÖÜ0-9_-]+)$/);
            var after = slate_1.Editor.after(editor, start);
            var afterRange = slate_1.Editor.range(editor, start, after);
            var afterText = slate_1.Editor.string(editor, afterRange);
            var afterMatch = afterText.match(/^(\s|$)/);
            if (beforeMatch && afterMatch) {
                setTarget(beforeRange !== null && beforeRange !== void 0 ? beforeRange : null);
                setSearch(beforeMatch[1]);
                setIndex(0);
                return;
            }
        }
        setTarget(null);
    }, []);
    react_1.useEffect(function () {
        if (disableMentions)
            return;
        if (target && possibleMentions.users.length > 0) {
            var el = ref.current;
            var domRange = slate_react_1.ReactEditor.toDOMRange(editor, target);
            var rect = domRange.getBoundingClientRect();
            el.style.top = rect.top + window.pageYOffset + 24 + "px";
            el.style.left = rect.left + window.pageXOffset + "px";
        }
    }, [possibleMentions.users.length, editor, index, search, target]);
    return {
        ref: ref,
        target: target,
        index: index,
        possibleMentions: possibleMentions,
        recentlyMentionedUsers: recentlyMentionedUsers,
        onKeyDown: onKeyDown,
        onChange: onChange,
        onMention: onMention,
    };
};
exports.useMentions = useMentions;
//# sourceMappingURL=mentions.js.map